package psoo.math.util;

import psoo.math.exception.VecteurIncorrect;

public class Vecteur {


	protected int dimension;        
	protected double[] composants;       

	public Vecteur() { dimension = 0;}
	
	public Vecteur(int n) {
		dimension = n;
		composants = new double[n];
	}

	public Vecteur(double[] donnees) {
		dimension = donnees.length;
		composants = donnees.clone();
	}
	
	public Vecteur(Vecteur v)
	{
		this(v.composants);
	}
	

	public int taille() {
		return dimension;
	}

	public double prodScalaire(Vecteur that) throws VecteurIncorrect {
		if (dimension != that.dimension) throw new VecteurIncorrect("Dimensions incorrectes");
		double s = 0.0;
		for (int i = 0; i < dimension; i++)
			s = s + (composants[i] * that.composants[i]);
		return s;
	}

	public double norme() throws VecteurIncorrect {
		return Math.sqrt(prodScalaire(this));
	}

	public double distanceA(Vecteur that) throws VecteurIncorrect {
		if (dimension != that.dimension) throw new VecteurIncorrect("Dimensions incorrectes");
		return moins(that).norme();
	}

	public Vecteur plus(Vecteur that) throws VecteurIncorrect {
		if (dimension != that.dimension) throw new VecteurIncorrect("Dimensions incorrectes");
		Vecteur c = new Vecteur(dimension);
		for (int i = 0; i < dimension; i++)
			c.composants[i] = composants[i] + that.composants[i];
		return c;
	}

	public Vecteur moins(Vecteur that) throws VecteurIncorrect {
		if (dimension != that.dimension) throw new VecteurIncorrect("Dimensions incorrectes");
		Vecteur c = new Vecteur(dimension);
		for (int i = 0; i < dimension; i++)
			c.composants[i] = composants[i] - that.composants[i];
		return c;
	}

	public double coordCartesienne(int i) {
		return composants[i];
	}

	public Vecteur fois(double facteur) {
		Vecteur c = new Vecteur(dimension);
		for (int i = 0; i < dimension; i++)
			c.composants[i] = facteur * composants[i];
		return c;
	}


	public Vecteur direction() throws VecteurIncorrect {
		if (norme() == 0.0) throw new VecteurIncorrect("Vecteur nul !");
		return fois(1.0 / this.norme());
	}

	public String toString() {
		String s = "(";
		for (int i = 0; i < dimension; i++) {
			s += composants[i];
			if (i < dimension-1) s+= ", "; 
		}
		return s + ")";
	}

	public static void main(String[] args) {
		double[] x0 = { 1.0, 2.0, 3.0, 4.0 };
		double[] y0 = { 5.0, 5.0, 3.0, 2.0 };

		Vecteur x = new Vecteur(x0);
		Vecteur y = new Vecteur(y0);

		System.out.println("x        =  " + x);
		System.out.println("y        =  " + y);
		try {
			System.out.println("x + y    =  " + x.plus(y));
			System.out.println("10x      =  " + x.fois(10.0));
			System.out.println("|x|      =  " + x.norme());
			System.out.println("<x, y>   =  " + x.prodScalaire(y));
			System.out.println("|x - y|  =  " + x.moins(y).norme());
		} catch (VecteurIncorrect e) {
			e.printStackTrace();
		}

	}
}
